#
#  gitrelease.rb  --  Read Git configuration
#

require "time"

module GitRelease

  class Config

    class <<self

      def config_bool *args
        args.map! { |a| [ a, :"#{a}?"] }
        def_config *args do |r|
          r !~ /^($|0+$|n|off|false)/i
        end
      end
      alias config_boole config_bool

      def config_int *args
        def_config *args do |r|
          Integer r
        end
      end
      alias config_integer config_int

      def config_str *args
        def_config *args do |r|
          r
        end
      end
      alias config config_str

      def config_time *args
        def_config *args do |r|
          Time.parse r
        end
      end

      def config_re *args
        def_config *args do |r|
          Regexp.new r
        end
      end
      alias config_regexp config_re

      private

      def def_config *args, &block
        args.each { |a,n|
          define_method n||a do
            read_config a, &block
          end
        }
      end

    end

    private

    def read_config a
      r = `git config #{self.class::CONFIG_SECTION}.#{a}`
      $?.success? or return
      r.chomp!
      yield r
    end

  end

  class Reference

    attr_reader :full, :type, :short

    def initialize ref
      ref =~ %r:refs/(\w+)/: or raise "Illegal reference: #{ref}."
      @full, @type, @short = ref, $1, $'
    end

    def to_s
      @full
    end

  end

end

