#
#  gitrelease/hooks.rb  --  Git Hook
#

require "supplement"

require "gitrelease"


module GitRelease

  class Hook < Config

    ZERO = "0000000000000000000000000000000000000000"

    def initialize
      ENV[ "GIT_DIR"] == "." or
        raise "Don't run this script from the command line."
      check_description
    end

    class <<self

      def run *args
        i = new *args
        i.execute
      rescue
        raise if $stderr.tty?
        m = $!.message
        if m[ $/] then
          $stderr.puts "*** #{$!.class}:"
          m.each_line { |l|
            $stderr.puts "    #{l}"
          }
        else
          $stderr.puts "*** #$! (#{$!.class})"
        end
        exit 1
      end

    end

    private

    DESCRIPTION = "description"

    def check_description
      File.open DESCRIPTION do |f|
        f.each_line { |l|
          case l
            when /^Unnamed repository/, /^$/ then
              raise "Project description file hasn't been set."
          end
          return
        }
      end
    end

  end

end

