#
#  gitrelease/hooks/update.rb  --  Git Update Hook
#

require "gitrelease/hooks"


module GitRelease

  # == Config
  #
  # hooks.allowunannotated
  # hooks.allowdeletetag
  # hooks.allowmodifytag
  # hooks.allowdeletebranch
  # hooks.denycreatebranch
  #
  class Update < Hook

    CONFIG_SECTION = "hooks"

    config_bool :allowunannotated, :allowdeletetag, :allowmodifytag
    config_bool :allowdeletebranch, :denycreatebranch

    def initialize
      super
      refname, oldrev, newrev = *$*
      refname && oldrev && newrev or
        raise "usage: #$0 <ref> <oldrev> <newrev>"
      @ref = Reference.new refname
      @oldrev = oldrev.nil_if ZERO
      @newrev = newrev.nil_if ZERO
    end

    def execute
      m = begin
        n = newrev_type
        method :"do_#{@ref.type}_#{n}"
      rescue NameError
        raise "Update hook: unknown update to ref #@ref of type #{n}."
      end
      m.call
    end

    private

    def do_tags_commit
      return if allowunannotated?
      raise <<-EOT
The un-annotated tag, #@ref, is not allowed in this repository.
Use 'git tag [ -a | -s ]' for tags you want to propagate.
      EOT
    end

    def do_tags_tag
      return if allowmodifytag?
      `git rev-parse -q #@ref 2>/dev/null`
      not $?.success? or raise <<-EOT
Tag #@ref already exists.
Modifying a tag is not allowed in this repository.
      EOT
    end

    def do_tags_delete
      return if allowdeletetag?
      raise <<-EOT
Deleting a tag is not allowed in this repository.
      EOT
    end

    def do_heads_commit
      return if @oldrev or not denycreatebranch?
      raise <<-EOT
Creating a branch is not allowed in this repository.
      EOT
    end

    def do_heads_delete
      return if allowdeletebranch?
      raise <<-EOT
Deleting a branch is not allowed in this repository.
      EOT
    end

    def do_remotes_commit
    end

    def do_remotes_delete
      return unless allowdeletebranch?
      raise <<-EOT
Deleting a tracking branch is not allowed in this repository.
      EOT
    end


    def newrev_type
      if @newrev then
        r = `git cat-file -t #@newrev`
        r.chomp!
        r
      else
        "delete"
      end
    end

  end

end

